from hamcrest.core.base_matcher import BaseMatcher
from jsonschema import Draft7Validator, exceptions, RefResolutionError
from jsonschema.exceptions import UnknownType


class MatchesJsonSchema(BaseMatcher):

    def __init__(self, json_schema: dict):
        self.json_schema = json_schema
        self.error = ''

    def _matches(self, instance):
        try:
            self.validator = Draft7Validator(self.json_schema)
            self.validator.validate(instance)
            return True

        except exceptions.ValidationError:
            for error in sorted(self.validator.iter_errors(instance), key=str):
                self.error += f"\n{error}\n\n------------"
            return False

        except (AttributeError, RefResolutionError, KeyError, UnknownType) as attr_e:
            raise Exception(f" Schema is invalid:\n\n{attr_e}\n\n{self.json_schema}")

        # except RefResolutionError as ref_e:
        #     raise RefResolutionError(f" Schema is invalid:\n\n{ref_e}\n\n{self.json_schema}")

    def describe_to(self, description):
        title = self.json_schema.get("title", None)
        title = f' "{title}"' if title else ''
        description.append_text(f'\n     JSON object should match schema{title}')

    def describe_mismatch(self, item, mismatch_description):
        mismatch_description.append_text(f'\n     mismatches occurred: \n{self.error}')


def matches_json_schema(json_schema: dict):
    return MatchesJsonSchema(json_schema)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="json-schema-matchers",
    version="0.1.3",
    author="Jamal Zeinalov",
    author_email="jamal.zeynalov@gmail.com",
    description="PyHamcrest matchers extension for json schema validation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JamalZeynalov/json-schema-matchers",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'jsonschema>=3.2.0',
        'PyHamcrest>=2.0.2'
    ],
    python_requires='>=3.6',
)
